/****************************************************************************
** Meta object code from reading C++ file 'setprimaryuseridjob.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../gpgmeqt/src/setprimaryuseridjob.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'setprimaryuseridjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob_t {
    QByteArrayData data[7];
    char stringdata0[84];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob_t qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QGpgME::SetPrimaryUserIDJob"
QT_MOC_LITERAL(1, 28, 6), // "result"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 12), // "GpgME::Error"
QT_MOC_LITERAL(4, 49, 5), // "error"
QT_MOC_LITERAL(5, 55, 14), // "auditLogAsHtml"
QT_MOC_LITERAL(6, 70, 13) // "auditLogError"

    },
    "QGpgME::SetPrimaryUserIDJob\0result\0\0"
    "GpgME::Error\0error\0auditLogAsHtml\0"
    "auditLogError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGpgME__SetPrimaryUserIDJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   29,    2, 0x06 /* Public */,
       1,    2,   36,    2, 0x26 /* Public | MethodCloned */,
       1,    1,   41,    2, 0x26 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, 0x80000000 | 3,    4,    5,    6,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void QGpgME::SetPrimaryUserIDJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SetPrimaryUserIDJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->result((*reinterpret_cast< const GpgME::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const GpgME::Error(*)>(_a[3]))); break;
        case 1: _t->result((*reinterpret_cast< const GpgME::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->result((*reinterpret_cast< const GpgME::Error(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SetPrimaryUserIDJob::*)(const GpgME::Error & , const QString & , const GpgME::Error & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SetPrimaryUserIDJob::result)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QGpgME::SetPrimaryUserIDJob::staticMetaObject = { {
    QMetaObject::SuperData::link<Job::staticMetaObject>(),
    qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob.data,
    qt_meta_data_QGpgME__SetPrimaryUserIDJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGpgME::SetPrimaryUserIDJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGpgME::SetPrimaryUserIDJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGpgME__SetPrimaryUserIDJob.stringdata0))
        return static_cast<void*>(this);
    return Job::qt_metacast(_clname);
}

int QGpgME::SetPrimaryUserIDJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QGpgME::SetPrimaryUserIDJob::result(const GpgME::Error & _t1, const QString & _t2, const GpgME::Error & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
